import sys, os
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTrajectory

def expFit(t, tau, y0, yf):

	return (yf - y0) * (1 - np.exp(-t / tau)) + y0

SSMALL = 4
SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]

lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]
flexSymbol = ["x", "o", "^"]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]
vfs = [0.05]

numRuns = 2

# Figures
traceFig = plt.figure(figsize=(1.693,1.693))
finalFig1 = plt.figure(figsize=(1.693,1.693))
finalFig2 = plt.figure(figsize=(1.693,1.693))

vIndex = 0		
for vf in vfs:

	lIndex = 0
	for lOn2R in lOn2Rs:

		linkerDl = lOn2R * 2 * radius
		linkerK = kbt / np.power(linkerDl, 2.0)
		Lc = (monomersPerPolymer - 1) * (2 * radius + linkerL + linkerDl) + 2 * radius
		
		fds = np.array([0.0 for model in models])
		fdsErr = np.array([0.0 for model in models])
		turns = np.array([0.0 for model in models])
		turnsErr = np.array([0.0 for model in models])

		mIndex = 0
		for model in models:
	
			fdRun = []
			fdRunErr = []
			turnRun = []
			turnRunErr = []
			for nr in range(numRuns):
			
				# Read
				iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.xml" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				inp = BioNetInput.BioNetInput()
				inp.load(iFname)
				
				tFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntrj" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				traj = BioNetTrajectory.BioNetTrajectory()
				traj.loadFinalFrame(tFname)
				
				# Get fractal dimensions
				threshold = radius
				radii = np.array([n.protein.radius for n in inp.node])
				fdPair, data = traj.frame[-1].calculateFractalDimension_BoxCount(inp.box.dimension, threshold, 2, radius = radii, periodic=True, dense=True)
				X = data[0]
				Y = data[1]
				func = data[2]
				popt = data[3]
				pcov = data[4]
				
				turn = popt[2]
				turnErr = np.sqrt(pcov[2][2])

				# Plot fractal fits as examples
				if (nr == 0):
					plt.figure(traceFig.number)
					plt.plot(X, Y ,"xC%d" % (mIndex), markersize = 3, linewidth=1, label="%s" % (modelNames2[mIndex]))
					plt.plot(X, func(X, *popt) ,"--C%d" % (mIndex), markersize = 3, linewidth=1)

				fdRun.append(fdPair[0])
				fdRunErr.append(fdPair[1])
				turnRun.append(inp.box.dimension[0] * np.power(2,-turn))
				turnRunErr.append(np.log(2) * turnRun[-1] * turnErr)


			fdsErr[mIndex] = np.std(fdRun) / np.sqrt(numRuns) + np.mean(fdRunErr)
			fds[mIndex] = np.mean(fdRun)

			turnsErr[mIndex] = np.std(turnRun) / np.sqrt(numRuns) + np.mean(turnRunErr)
			turns[mIndex] = np.mean(turnRun)

#			if (mIndex == 0 and lIndex == 2) or (mIndex == 5 and lIndex == 0):
#				print("Fd: %f pm %f" % (fds[mIndex], fdsErr[mIndex]))
#				print("FdTP: %f pm %f" % (turns[mIndex], turnsErr[mIndex]))

			mIndex += 1
			
		# Plot
		plt.figure(finalFig1.number)
		plt.errorbar(range(fds.size), fds, yerr=fdsErr, fmt="-" + flexSymbol[lIndex] + "C%d" % (lIndex), markersize=3, linewidth=1, label="%s" % (stiffnessNames[lIndex]))
	
		plt.figure(finalFig2.number)
		plt.errorbar(range(turns.size), turns, yerr=turnsErr, fmt="-" + flexSymbol[lIndex] + "C%d" % (lIndex), markersize=3, linewidth=1, label="%s" % (stiffnessNames[lIndex]))
	
		# Prettify
		plt.figure(traceFig.number)
		plt.xlabel("Num Boxes (1D)", fontsize=SMALL)
		plt.ylabel("Num Populated (3D)", fontsize=SMALL)
		plt.legend(loc=0, fontsize=SSMALL)


		for tick in plt.gca().xaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)
		for tick in plt.gca().yaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)

		t = np.array([0.0,2.0,4.0,6.0])
		l = np.array([str(bit) for bit in t])
		plt.xticks(t,l)

		t = np.array([0.0,5.0,10.0,15.0])
		l = np.array([str(bit) for bit in t])
		plt.yticks(t,l)


		plt.tight_layout()

		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Kinetic_FractalDimensionTrace.png" % (lIndex, stiffnessNames[lIndex], vf)
		plt.savefig(oFname, dpi=300)
		plt.clf()
		
		lIndex += 1
	
	# Prettify
	plt.figure(finalFig1.number)
	plt.xticks(range(fds.size), modelNames2[:fds.size])

	t = np.array([2.1,2.15,2.2,2.25])
	l = np.array([str(bit) for bit in t])
	plt.yticks(t,l)


	plt.xlabel("Model", fontsize=SMALL)
	plt.ylabel("Fractal Dimension", fontsize=SMALL)
	plt.legend(loc=0, fontsize=SSMALL)

	for tick in plt.gca().xaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)
	for tick in plt.gca().yaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)

	plt.tight_layout()
	
	oFname = "../results/AnalyticalPolymerGel_vf%.3f_Kinetic_FinalFractalDimensions.png" % (vf)
	plt.savefig(oFname, dpi=300)
	plt.clf()

	plt.figure(finalFig2.number)
	plt.xticks(range(turns.size), modelNames2[:turns.size])

	t = np.array([20,25,30,35,40])
	l = np.array([str(bit) for bit in t])
	plt.yticks(t,l)

	plt.xlabel("Model", fontsize=SMALL)
	plt.ylabel("Fractal Upper Limit (nm)", fontsize=SMALL)
	plt.legend(loc=0, fontsize=SSMALL)

	for tick in plt.gca().xaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)
	for tick in plt.gca().yaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)

	plt.tight_layout()
	
	oFname = "../results/AnalyticalPolymerGel_vf%.3f_Kinetic_FinalFractalDimensionsTurningPoints.png" % (vf)
	plt.savefig(oFname, dpi=300)
	plt.clf()

	vIndex += 1
