import sys, os
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTrajectory, BioNetMeasurement

def expFit(t, tau, y0, yf):

	return (yf - y0) * (1 - np.exp(-t / tau)) + y0

SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]

lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]

vfs = [0.05,0.07,0.1]

modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]
vfs = [0.05]

numRuns = 2

# Figures
traceFig = plt.figure(figsize=(3.2,3.2))
traceFigTurn = plt.figure(figsize=(3.2,3.2))

vIndex = 0		
for vf in vfs:

	lIndex = 0
	for lOn2R in lOn2Rs:

		linkerDl = lOn2R * 2 * radius
		linkerK = kbt / np.power(linkerDl, 2.0)
		Lc = (monomersPerPolymer - 1) * (2 * radius + linkerL + linkerDl) + 2 * radius
		
		mIndex = 0
		for model in models:

			# Time trace (every 50 frames)
			stepsize = 200

			for nr in range(numRuns):
			
				time = []
				fdTrace = []
				fdErrTrace = []

				turnTrace = []
				turnErrTrace = []
			
				# Read
				iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.xml" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				inp = BioNetInput.BioNetInput()
				inp.load(iFname)
	
				tFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntrj" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				traj = BioNetTrajectory.BioNetTrajectory()

				mFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bnms" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				meas = BioNetMeasurement.BioNetMeasurement()
				meas.load(mFname)

				traj.load(tFname, rate=stepsize)

				# Get fractal dimensions
				for frameIndex in range(traj.numFrames):			
					threshold = radius
					radii = np.array([n.protein.radius for n in inp.node])
					fdPair, data = traj.frame[frameIndex].calculateFractalDimension_BoxCount(inp.box.dimension, threshold, 2, radius = radii, periodic=True, dense=True)

					popt = data[3]
					pcov = data[4]
					turn = popt[2]
					turnErr = np.sqrt(pcov[2][2])

					time.append(traj.frame[frameIndex].time)
					fdTrace.append(fdPair[0])
					fdErrTrace.append(fdPair[1])

					turnTrace.append(inp.box.dimension[0] * np.power(2,-turn))
					turnErrTrace.append(np.log(2) * turnTrace[-1] * turnErr)

				# Plot
				time = np.array(time) / 1000.0
				plt.figure(traceFig.number)
				if nr == 0:
					plt.errorbar(time, fdTrace, yerr=fdErrTrace, fmt="-xC%d" % (mIndex), label=r"M$_%d$" % (mIndex))
				else:
					plt.errorbar(time, fdTrace, yerr=fdErrTrace, fmt="-xC%d" % (mIndex))

				plt.figure(traceFigTurn.number)
				if nr == 0:
					plt.errorbar(time, turnTrace, yerr=turnErrTrace, fmt="-xC%d" % (mIndex), label=r"M$_%d$" % (mIndex))
				else:
					plt.errorbar(time, turnTrace, yerr=turnErrTrace, fmt="-xC%d" % (mIndex))

			mIndex += 1
			
		# Prettify
		plt.figure(traceFig.number)
#		plt.ylim([-0.1,1.6])
		plt.xlabel(r"Time ($\mu$s)", fontsize=MEDIUM)
		plt.ylabel("Fractal Dimension", fontsize=MEDIUM)
		plt.legend(loc=0, fontsize=SMALL)
		plt.tight_layout()

		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Kinetic_FractalDimensionOverTime.png" % (lIndex, stiffnessNames[lIndex], vf)
		plt.savefig(oFname, dpi=100)
		plt.clf()

		# Prettify
		plt.figure(traceFigTurn.number)
#		plt.ylim([-0.1,1.6])
		plt.xlabel(r"Time ($\mu$s)", fontsize=MEDIUM)
		plt.ylabel("Fractal Upper Limit (nm)", fontsize=MEDIUM)
		plt.legend(loc=0, fontsize=SMALL)
		plt.tight_layout()

		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Kinetic_FractalUpperLimitOverTime.png" % (lIndex, stiffnessNames[lIndex], vf)
		plt.savefig(oFname, dpi=100)
		plt.clf()
		
		lIndex += 1

	vIndex += 1
