import sys, os, copy
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTrajectory, BioNetTopology

def calculateElasticForce(f, nIndex, sIndex, inp, bondK, bondL):

	# Positions pair
	p = [f.sPos[nIndex[i]][sIndex[i]] for i in range(2)]
	
	# Geometry
	r = p[1] - p[0]
	
	# Boundary conditions
	trans = np.array([0.0,0.0,0.0])
	for i in range(3):
		if r[i] > inp.box.dimension[i] / 2.0:
			trans[i] = inp.box.dimension[i]
		elif r[i] < -inp.box.dimension[i] / 2.0:
			trans[i] = -inp.box.dimension[i]
			
	p[0] += trans
	r = p[1] - p[0]
	
	# Force calcs
	rMag = np.linalg.norm(r)
	rHat = r / rMag
	
	force = bondK * (rMag - bondL) * rHat
	
	return force
	
# This method calculates the localised stresses emerging from steric and elastic (bond) interactions
def calculateLocalForces(inp, traj, top, monomersPerPolymer, steric = True):

	# Constants
	xHat = np.array([1.0,0.0,0.0])
	yHat = np.array([0.0,1.0,0.0])
	zHat = np.array([0.0,0.0,1.0])
	
	# Node forces
	polymerNodeForce = np.array([[0.0, 0.0, 0.0] for j in range(monomersPerPolymer)])
	polymerNodeForceErr = np.array([[0.0, 0.0, 0.0] for j in range(monomersPerPolymer)])

	# Build connection map
	nodeConnections = [[] for i in range(inp.numNodes)]
	for c in inp.connection:

		nodeConnections[c.nIndex[0]].append([[c.nIndex[0], c.nIndex[1]], [c.sIndex[0], c.sIndex[1]], c.bond])
		nodeConnections[c.nIndex[1]].append([[c.nIndex[1], c.nIndex[0]], [c.sIndex[1], c.sIndex[0]], c.bond])
	
	# Determine forces one node at a time
	nsIndex = 0
	for nodeSet in nodeConnections:
		
		# Containers			
		nodeForce = np.array([[0.0 for i in range(3)] for j in range(3)])
		nodeForceErr = np.array([[0.0 for i in range(3)] for j in range(3)])

		# Build site index mappings
		if inp.node[nsIndex].protein.numSites == 2:
			indices = [[0,1],[-1,-1],[-1,-1]]
		elif inp.node[nsIndex].protein.numSites == 4:
			indices = [[0,3],[2,1],[-1,-1]]
		elif inp.node[nsIndex].protein.numSites == 6:
			indices = [[0,5],[2,1],[4,3]]
		else:
			print("Broken")
			sys.exit()
					
		# Average total stress on node over frames
		fIndex = 0
		for f in traj.frame:

			# Get local coordinate system (from node rotation)
			eHat = [np.dot(f.rMat[nsIndex], xHat), np.dot(f.rMat[nsIndex], yHat), np.dot(f.rMat[nsIndex], zHat)]			
			
			# Separate all forces by direction
			forces = [[np.array([0.0,0.0,0.0]) for i in range(2)] for j in range(3)]
			for connection in nodeSet:
				
				# Get connectivities and local force direction
				nIndex = connection[0]
				sIndex = connection[1]

				# Get bond parameters
				bondK = connection[2].k
				bondL = connection[2].l
				
				# Do all directions
				for i in range(3):
					for j in range(2):
						if sIndex[0] == indices[i][j]:
						
							# Cartesian
							forces[i][j] = calculateElasticForce(f, nIndex, sIndex, inp, bondK, bondL)

							# Local
							forces[i][j] = np.dot(forces[i][j], eHat[i]) * eHat[i]
							
			# The forces that would be experience like thos in AFM experiments is the average of difference in these forces
			AFMForce = [np.array([0.0,0.0,0.0]) for i in range(3)]			
			
			for i in range(3):
				AFMForce[i] = 0.5 * (forces[i][1] - forces[i][0])
				nodeForce[i] += AFMForce[i]
				nodeForceErr[i] += AFMForce[i] ** 2
			fIndex += 1
			
		# Averages and add to the polymer node bit
		for i in range(3):
			nodeForce[i] /= traj.numFrames
			nodeForceErr[i] /= traj.numFrames
			nodeForceErr[i] = np.sqrt((nodeForceErr[i] - nodeForce[i]**2) / traj.numFrames)
			
			# Magnitudes
			fMag = np.linalg.norm(nodeForce[i])
			fMagErr = 0.0
			if fMag > 0.0:
				for j in range(3):
					fMagErr += ((nodeForce[i][j] / fMag) * nodeForceErr[i][j])**2
				fMagErr = np.sqrt(fMagErr)

			polymerNodeForce[nsIndex % monomersPerPolymer][i] += fMag
			polymerNodeForceErr[nsIndex % monomersPerPolymer][i] += fMagErr
						
		nsIndex += 1

	# Averages
	polymerNodeForce /= (inp.numNodes / monomersPerPolymer)
	polymerNodeForceErr /= (inp.numNodes / monomersPerPolymer)

	return polymerNodeForce, polymerNodeForceErr

SSMALL = 4
SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]

numModels = len(models)
lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]
vfs = [0.05]

numRuns = 1

numBars = monomersPerPolymer + 2
barWidth = 1.0 / numBars

# Figs
forceFig = plt.figure(figsize=(1.693,1.693))

#
# Important!
#
frameRate = 1


vIndex = 0		
for vf in vfs:

	lIndex = 0
	for lOn2R in lOn2Rs:



		stiffnessName = stiffnessNames[lIndex]
				
		mIndex = 0
		for model in models:

			modelName = "GC%d-A" % (mIndex)
			modelString = r"M$_%d$" % (mIndex)
			
			noPlot = False
			for nr in range(numRuns):
				
				# Read (and set nans if it doesn't exist)
				iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Stress_Run%d.xml" % (modelName, stiffnessName, vf, nr)
				inp = BioNetInput.BioNetInput()
				inp.load(iFname)

				tFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Stress_Run%d.bntrj" % (modelName, stiffnessName, vf, nr)
				if not os.path.exists(tFname):
					noPlot = True
					break
					
				traj = BioNetTrajectory.BioNetTrajectory()
				traj.load(tFname, rate=frameRate, inScript=inp)

				tpFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntop" % (modelName, stiffnessName, vf, nr)
				top = BioNetTopology.BioNetTopology()
				top.loadFinalFrame(tpFname)
				
				# Calculate forces on each node, and in each cartesian direction
				polymerNodeForce, polymerNodeForceErr = calculateLocalForces(inp, traj, top, monomersPerPolymer)

			if noPlot:
				mIndex += 1
				continue
				
			# Plot each force
#			print(polymerNodeForce)
#			sys.exit()
			xPos = mIndex + (np.array([i for i in range(monomersPerPolymer)]) + 1.5) * barWidth
			
			y = [None,None,None]
			yErr = [None,None,None]
			for i in range(3):
				y[i] = np.array([p[i] for p in polymerNodeForce])
				yErr[i] = np.array([p[i] for p in polymerNodeForceErr])
				
				if i == 0:
					if mIndex == 0:
						plt.bar(xPos, y[i], barWidth, yerr=yErr[i], edgecolor="black", linewidth = 1, color="C%d" % (i), label="Axial")

					else:
						plt.bar(xPos, y[i], barWidth, yerr=yErr[i], edgecolor="black", linewidth = 1, color="C%d" % (i))
				elif i == 1:
					if mIndex == 0:
						plt.bar(xPos, y[i], barWidth, yerr=yErr[i], edgecolor="black", linewidth = 1, bottom = y[i-1], color="C%d" % (i), label="Radial %d" % (i))
					else:
						plt.bar(xPos, y[i], barWidth, yerr=yErr[i], edgecolor="black", linewidth = 1, bottom = y[i-1], color="C%d" % (i))
				else:
					if mIndex == 0:
						plt.bar(xPos, y[i], barWidth, yerr=yErr[i], edgecolor="black", linewidth = 1, bottom = y[i-1] + y[i - 2], color="C%d" % (i), label="Radial %d" % (i))
					else:
						plt.bar(xPos, y[i], barWidth, yerr=yErr[i], edgecolor="black", linewidth = 1, bottom = y[i-1] + y[i - 2], color="C%d" % (i))
			mIndex += 1
			
		# Prettify
		t = np.array([i for i in range(numModels)]) + 0.5
		l = [r"M$_%d$" % (i) for i in range(numModels)]
		plt.xticks(t,l)

		t = np.array([0,10,20,30, 40])
		l = np.array([str(bit) for bit in t])
		plt.yticks(t,l)

		plt.ylim([0,45])
		plt.xlabel("Model", fontsize=SMALL)
		plt.ylabel("Force (pN)", fontsize=SMALL)
		plt.legend(loc=0, fontsize=SSMALL, ncol=1)

		for tick in plt.gca().xaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)
		for tick in plt.gca().yaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)

		plt.tight_layout()			
		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Stress_LocalForces.png" % (lIndex, stiffnessName, vf)
		plt.savefig(oFname, dpi=300)
		plt.clf()
					
		lIndex += 1

	vIndex += 1
