import sys, os, copy
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTrajectory, BioNetTopology

# This method calculates the system wide internal stress emerging from steric and elastic (bond) interactions
def calculateTotalStress(inp, traj, top, steric = True):

	# System stress (3x3 matrix)
	totalStress = np.array([[0.0 for i in range(3)] for j in range(3)])
	totalStressErr = np.array([[0.0 for i in range(3)] for j in range(3)])
	
	# Data we'll need
	vol = np.product(inp.box.dimension)
	stericK = 1560.5196694	# From code
	radius = 2.5	# Standardised
	
	# Reversing the sums (average system wide stresses over all frames)
	fIndex = 0
	for f in traj.frame:
	
		frameStress = np.array([[0.0 for i in range(3)] for j in range(3)])
	
		# To get total elastic stress, sum contributions over all connections. Assume steric only applies between connected pairs?
		cIndex = 0
		for c in inp.connection:

			# Get bond parameters
			bondK = c.bond.k
			bondL = c.bond.l
			
			# Get connectivities
			nIndex = c.nIndex
			sIndex = c.sIndex
			
			# Data containers for this pairing (we need to keep track of two stress tensors, one for each site)
			siteStress = np.array([[[0.0 for i in range(3)] for j in range(3)] for k in range(2)])
		
			# Positions pair
			p = [f.sPos[nIndex[i]][sIndex[i]] for i in range(2)]
			
			# Geometry
			r = p[1] - p[0]
			
			# Boundary conditions
			trans = np.array([0.0,0.0,0.0])
			for i in range(3):
				if r[i] > inp.box.dimension[i] / 2.0:
					trans[i] = inp.box.dimension[i]
				elif r[i] < -inp.box.dimension[i] / 2.0:
					trans[i] = -inp.box.dimension[i]
					
			p[0] += trans
			r = p[1] - p[0]
			
			# Force calcs
			rMag = np.linalg.norm(r)
			rHat = r / rMag
			
			force = bondK * (rMag - bondL) * rHat
			
			# Stresses for both nodes (errors are not independent (per site), so take errors over bonds only)
			siteStress = [np.outer(force, p[0]), np.outer(-force, p[1])]
			frameStress += siteStress[0] + siteStress[1]
			
			# Reset (just in case python is weird)
			p[0] -= trans

			# Steric for node stresses and frame stresses
			if steric:
			
				# Positions pair
				p = [f.pos[nIndex[i]] for i in range(2)]
				
				# Geometry
				r = p[1] - p[0]
				
				# Boundary conditions
				trans = np.array([0.0,0.0,0.0])
				for i in range(3):
					if r[i] > inp.box.dimension[i] / 2.0:
						trans[i] = inp.box.dimension[i]
					elif r[i] < -inp.box.dimension[i] / 2.0:
						trans[i] = -inp.box.dimension[i]
						
				p[0] += trans
				r = p[1] - p[0]
				
				# Force calcs
				rMag = np.linalg.norm(r)
				if rMag < 2 * radius:
				
					rHat = r / rMag
					rOnTwoR = rMag / (2 * radius)
					force = -((8.0 / 3.0) * stericK * radius * (2 + rOnTwoR) * (1 + rOnTwoR) * (1 - rOnTwoR) ** 3) * rHat
					siteStress = [np.outer(force, p[0]), np.outer(-force, p[1])]
					frameStress += siteStress[0] + siteStress[1]

			cIndex += 1
		
		# Finalise frame stress (reduce by volume, convert to kPa, positive stress means pushing outwards!)
		frameStress *= 1000.0 / vol

		# Add to total
		totalStress += frameStress
		totalStressErr += frameStress**2

	totalStress /= traj.numFrames
	totalStressErr /= traj.numFrames
	totalStressErr = np.sqrt((totalStressErr - totalStress**2) / traj.numFrames)	

	return totalStress, totalStressErr
	
SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = ["M%d" % (i) for i in range(len(models))]

lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]
vfs = [0.05]
strains = [0.0, 0.01]
numRuns = 1

# Figs
stressFig = plt.figure(figsize=(3.2,3.2))

#
# Important!
#
frameRate = 100

vIndex = 0		
for vf in vfs:
	
	strIndex = 0
	for strain in strains:
		lIndex = 0
		for lOn2R in lOn2Rs:

			stiffnessName = stiffnessNames[lIndex]


			# Data containers
			tStress = []
			tStressErr = []
			nStress = [[] for i in range(monomersPerPolymer)]
			nStressErr = [[] for i in range(monomersPerPolymer)]
					
			mIndex = 0
			for model in models:

				modelName = "GC%d-A" % (mIndex)
				modelString = "M%d" % (mIndex)

				for nr in range(numRuns):
					
					# Read (and set nans if it doesn't exist)
					if strain == 0.0:
						iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Stress_Run%d.xml" % (modelName, stiffnessName, vf, nr)
						tFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Stress_Run%d.bntrj" % (modelName, stiffnessName, vf, nr)
						tpFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntop" % (modelName, stiffnessName, vf, nr)					
					else:
						iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Strain%.2f_Stress_Run%d.xml" % (modelName, stiffnessName, vf, strain, nr)
						tFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Strain%.2f_Stress_Run%d.bntrj" % (modelName, stiffnessName, vf, strain, nr)
						tpFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntop" % (modelName, stiffnessName, vf, nr)
			
					# Check existence
					if not os.path.exists(tFname):
						stressAvg = np.nan
						stressError = np.nan
						continue

					inp = BioNetInput.BioNetInput()
					inp.load(iFname)
					traj = BioNetTrajectory.BioNetTrajectory()
					traj.load(tFname, rate=frameRate, inScript=inp)
					top = BioNetTopology.BioNetTopology()
					top.loadFinalFrame(tpFname)
					
					# Calculate stresses
					totalStress, totalStressErr = calculateTotalStress(inp, traj, top, steric=False)				
					
					# Plot average of the diagonal (as all shear cancels, as expected)
					stressAvg = np.mean(np.diag(totalStress))
					stressError = np.mean(np.diag(totalStressErr)) + np.std(np.diag(totalStressErr)) / np.sqrt(3)

						
				tStress.append(stressAvg)
				tStressErr.append(stressError)

				mIndex += 1
				
			# Plot
			plt.figure(stressFig.number)
			plt.errorbar(range(len(tStress)), tStress, yerr=tStressErr, fmt="-xC%d" % (lIndex), label="%s" % (stiffnessName) + r", $\epsilon$ = %d%%" % (strain * 100))
			
			lIndex += 1
		strIndex += 1
			
	# Prettify (and add second axis)
	plt.figure(stressFig.number)
	plt.xticks(range(len(tStress)), modelNames2[:len(tStress)])
	plt.xlabel("Model", fontsize=MEDIUM)
	plt.ylabel("Total Internal Stress (kPa)", fontsize=MEDIUM)
	plt.legend(loc=0, fontsize=SMALL)
	
#	ax1 = plt.gca()
#	ax2 = ax1.twinx() 
#	ax2.set_ylim(ax1.get_ylim())	
#	ax2.set_yticks(ax1.get_yticks())
#
#	ticklabels = np.array([float(b) for b in ax1.get_yticks()])
#	ticklabels *= 1e-3 * inp.box.dimension[0]**2
#	ax2.set_yticklabels(["%.1f" % (t) for t in ticklabels])
#	ax2.set_ylabel("Expected Internal Force (pN)", fontsize=MEDIUM)

	plt.tight_layout()
	
	oFname = "../results/AnalyticalPolymerGel_vf%.3f_TotalStresses.png" % (vf)
	plt.savefig(oFname, dpi=100)
	plt.clf()

	vIndex += 1
